from typing import get_type_hints

from dataclasses_json import dataclass_json

from grebble_flow.managment.processor import find_all_processors


def get_processors_info():
    result = []
    processors = find_all_processors()
    for processor in processors:
        item = {"name": processor.name}
        if hasattr(processor, "attributes"):
            item['attributes_schema'] = get_type_hints(processor.attributes)
            for key in item['attributes_schema']:
                item['attributes_schema'][key] = {
                    "type": item['attributes_schema'][key].__name__,
                }
        result.append(item)
    return result


def generate_package_info():
    result = {"processors": get_processors_info()}
    return result



import inspect
import sys

from grebble_flow.processors.base import BaseFlowProcessor


def import_processors():
    module = "processors"
    module_path = module

    if module_path in sys.modules:
        return sys.modules[module_path]

    return __import__(module_path, fromlist=[module])


def find_all_processors():
    import_processors()
    inspect.getmembers(import_processors())
    result = [cls for cls in BaseFlowProcessor.__subclasses__()]
    return result



import os
from pathlib import Path

import pkg_resources


def install_proto():
    import grpc_tools.protoc

    proto_include = pkg_resources.resource_filename(
        "grebble_flow", "proto/internal/v1/"
    )
    python_out = pkg_resources.resource_filename(
        "grebble_flow", "grpc/generated/internal"
    )
    for file in ["app.proto", "processor.proto"]:
        grpc_tools.protoc.main(
            [
                "grpc_tools.protoc",
                f"--proto_path={proto_include}",
                f"--python_out={python_out}",
                f"--grpc_python_out={python_out}",
                f"{proto_include}/{file}",
            ]
        )



import json

from grebble_flow.grpc.generated.internal.v1 import app_pb2_grpc
from grebble_flow.managment.info import generate_package_info


class AppService(app_pb2_grpc.ExternalAppServicer):
    def __init__(self, *args, **kwargs):
        pass

    def AppInfo(self, request, context):
        info = generate_package_info()

        result = app_pb2_grpc.AppInfoExternalResponse()
        result.processors.extend(
            [
                app_pb2_grpc.ProcessorExternalInfo(
                    name=processor["name"],
                    attributeSchema=json.dumps(processor["attributes_schema"])
                    if processor.get("attributes_schema")
                    else "",
                )
                for processor in info["processors"]
            ]
        )
        return result
